# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feature_store_serving.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from mlflow.protos.scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x65\x61ture_store_serving.proto\x12\x12\x66\x65\x61turestorecommon\x1a\x15scalapb/scalapb.proto\"\xdc\x03\n\x15OnlineStoreForServing\x12(\n\x05\x63loud\x18\x01 \x01(\x0e\x32\x19.featurestorecommon.Cloud\x12\x31\n\nstore_type\x18\x02 \x01(\x0e\x32\x1d.featurestorecommon.StoreType\x12\x1a\n\x12read_secret_prefix\x18\x03 \x01(\t\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x04 \x01(\x03\x12\x33\n\nmysql_conf\x18\x05 \x01(\x0b\x32\x1d.featurestorecommon.MySqlConfH\x00\x12<\n\x0fsql_server_conf\x18\x06 \x01(\x0b\x32!.featurestorecommon.SqlServerConfH\x00\x12\x39\n\rdynamodb_conf\x18\x07 \x01(\x0b\x32 .featurestorecommon.DynamoDbConfH\x00\x12\x39\n\rcosmosdb_conf\x18\x08 \x01(\x0b\x32 .featurestorecommon.CosmosDbConfH\x00\x12\x31\n\nquery_mode\x18\x15 \x01(\x0e\x32\x1d.featurestorecommon.QueryModeB\x0f\n\rextra_configs\"\'\n\tMySqlConf\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"+\n\rSqlServerConf\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"\x1e\n\x0c\x44ynamoDbConf\x12\x0e\n\x06region\x18\x01 \x01(\t\"#\n\x0c\x43osmosDbConf\x12\x13\n\x0b\x61\x63\x63ount_uri\x18\x01 \x01(\t\"j\n\x0e\x46\x65\x61tureDetails\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\tdata_type\x18\x02 \x01(\x0e\x32\x1c.featurestorecommon.DataType\x12\x19\n\x11\x64\x61ta_type_details\x18\x03 \x01(\t\"R\n\x11PrimaryKeyDetails\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\tdata_type\x18\x02 \x01(\x0e\x32\x1c.featurestorecommon.DataType\"T\n\x13TimestampKeyDetails\x12\x0c\n\x04name\x18\x01 \x01(\t\x12/\n\tdata_type\x18\x02 \x01(\x0e\x32\x1c.featurestorecommon.DataType\"\xe2\x02\n\x12OnlineFeatureTable\x12\x1a\n\x12\x66\x65\x61ture_table_name\x18\x01 \x01(\t\x12!\n\x19online_feature_table_name\x18\x02 \x01(\t\x12?\n\x0conline_store\x18\x03 \x01(\x0b\x32).featurestorecommon.OnlineStoreForServing\x12;\n\x0cprimary_keys\x18\x04 \x03(\x0b\x32%.featurestorecommon.PrimaryKeyDetails\x12\x18\n\x10\x66\x65\x61ture_table_id\x18\x05 \x01(\t\x12\x34\n\x08\x66\x65\x61tures\x18\x06 \x03(\x0b\x32\".featurestorecommon.FeatureDetails\x12?\n\x0etimestamp_keys\x18\x07 \x03(\x0b\x32\'.featurestorecommon.TimestampKeyDetails\"X\n\x17\x46\x65\x61tureTablesForServing\x12=\n\ronline_tables\x18\x01 \x03(\x0b\x32&.featurestorecommon.OnlineFeatureTable\"+\n\rMySqlMetadata\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"/\n\x11SqlServerMetadata\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\"B\n\x10\x44ynamoDbMetadata\x12\x0e\n\x06region\x18\x01 \x01(\t\x12\x11\n\ttable_arn\x18\x02 \x01(\t\x12\x0b\n\x03ttl\x18\x03 \x01(\x03\"K\n\x10\x43osmosDbMetadata\x12\x13\n\x0b\x61\x63\x63ount_uri\x18\x01 \x01(\t\x12\x15\n\rcontainer_uri\x18\x02 \x01(\t\x12\x0b\n\x03ttl\x18\x03 \x01(\x03\"\xa9\x04\n\x0bOnlineStore\x12(\n\x05\x63loud\x18\x01 \x01(\x0e\x32\x19.featurestorecommon.Cloud\x12\x31\n\nstore_type\x18\x02 \x01(\x0e\x32\x1d.featurestorecommon.StoreType\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x04 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x05 \x01(\x03\x12\x12\n\ncreator_id\x18\x06 \x01(\t\x12\x10\n\x04host\x18\x07 \x01(\tB\x02\x18\x01\x12\x10\n\x04port\x18\x08 \x01(\x05\x42\x02\x18\x01\x12\x1b\n\x13last_update_user_id\x18\t \x01(\t\x12;\n\x0emysql_metadata\x18\n \x01(\x0b\x32!.featurestorecommon.MySqlMetadataH\x00\x12\x44\n\x13sql_server_metadata\x18\x0b \x01(\x0b\x32%.featurestorecommon.SqlServerMetadataH\x00\x12\x41\n\x11\x64ynamodb_metadata\x18\x0c \x01(\x0b\x32$.featurestorecommon.DynamoDbMetadataH\x00\x12\x41\n\x11\x63osmosdb_metadata\x18\r \x01(\x0b\x32$.featurestorecommon.CosmosDbMetadataH\x00\x42\x15\n\x13\x61\x64\x64itional_metadata\"\xdf\x04\n\x13OnlineStoreDetailed\x12(\n\x05\x63loud\x18\x01 \x01(\x0e\x32\x19.featurestorecommon.Cloud\x12\x31\n\nstore_type\x18\x02 \x01(\x0e\x32\x1d.featurestorecommon.StoreType\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1a\n\x12\x63reation_timestamp\x18\x04 \x01(\x03\x12\x1e\n\x16last_updated_timestamp\x18\x05 \x01(\x03\x12\x12\n\ncreator_id\x18\x06 \x01(\t\x12\x10\n\x04host\x18\x07 \x01(\tB\x02\x18\x01\x12\x10\n\x04port\x18\x08 \x01(\x05\x42\x02\x18\x01\x12\x1b\n\x13last_update_user_id\x18\t \x01(\t\x12\x10\n\x08\x66\x65\x61tures\x18\n \x03(\t\x12\x1a\n\x12read_secret_prefix\x18\x0b \x01(\t\x12;\n\x0emysql_metadata\x18\x0c \x01(\x0b\x32!.featurestorecommon.MySqlMetadataH\x00\x12\x44\n\x13sql_server_metadata\x18\r \x01(\x0b\x32%.featurestorecommon.SqlServerMetadataH\x00\x12\x41\n\x11\x64ynamodb_metadata\x18\x0e \x01(\x0b\x32$.featurestorecommon.DynamoDbMetadataH\x00\x12\x41\n\x11\x63osmosdb_metadata\x18\x0f \x01(\x0b\x32$.featurestorecommon.CosmosDbMetadataH\x00\x42\x15\n\x13\x61\x64\x64itional_metadata\"t\n FeatureTablesForSageMakerServing\x12P\n\ronline_tables\x18\x01 \x03(\x0b\x32\x39.featurestorecommon.OnlineFeatureTableForSageMakerServing\"\xfe\x02\n%OnlineFeatureTableForSageMakerServing\x12\x1a\n\x12\x66\x65\x61ture_table_name\x18\x01 \x01(\t\x12!\n\x19online_feature_table_name\x18\x02 \x01(\t\x12H\n\x0conline_store\x18\x03 \x01(\x0b\x32\x32.featurestorecommon.OnlineStoreForSageMakerServing\x12;\n\x0cprimary_keys\x18\x04 \x03(\x0b\x32%.featurestorecommon.PrimaryKeyDetails\x12\x18\n\x10\x66\x65\x61ture_table_id\x18\x05 \x01(\t\x12\x34\n\x08\x66\x65\x61tures\x18\x06 \x03(\x0b\x32\".featurestorecommon.FeatureDetails\x12?\n\x0etimestamp_keys\x18\x07 \x03(\x0b\x32\'.featurestorecommon.TimestampKeyDetails\"\xbe\x01\n\x1eOnlineStoreForSageMakerServing\x12\x1d\n\x15\x63reation_timestamp_ms\x18\x01 \x01(\x03\x12\x39\n\rdynamodb_conf\x18\x02 \x01(\x0b\x32 .featurestorecommon.DynamoDbConfH\x00\x12\x31\n\nquery_mode\x18\x10 \x01(\x0e\x32\x1d.featurestorecommon.QueryModeB\x0f\n\rextra_configs\"\x83\x01\n\x1c\x46\x65\x61tureFunctionParameterInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\ttype_text\x18\x02 \x01(\t\x12\x11\n\ttype_json\x18\x03 \x01(\t\x12/\n\ttype_name\x18\x04 \x01(\x0e\x32\x1c.featurestorecommon.DataType\"\xdb\x01\n\x19\x46\x65\x61tureFunctionForServing\x12\x11\n\tfull_name\x18\x01 \x01(\t\x12\x46\n\x0cinput_params\x18\x02 \x03(\x0b\x32\x30.featurestorecommon.FeatureFunctionParameterInfo\x12/\n\tdata_type\x18\x03 \x01(\x0e\x32\x1c.featurestorecommon.DataType\x12\x16\n\x0e\x66ull_data_type\x18\x04 \x01(\t\x12\x1a\n\x12routine_definition\x18\x05 \x01(\t\"c\n\x1a\x46\x65\x61tureFunctionsForServing\x12\x45\n\x0e\x66\x66_for_serving\x18\x01 \x03(\x0b\x32-.featurestorecommon.FeatureFunctionForServing*$\n\x05\x43loud\x12\x07\n\x03\x41WS\x10\x01\x12\t\n\x05\x41ZURE\x10\x02\x12\x07\n\x03GCP\x10\x03*T\n\tStoreType\x12\x10\n\x0c\x41URORA_MYSQL\x10\x01\x12\x0e\n\nSQL_SERVER\x10\x02\x12\t\n\x05MYSQL\x10\x03\x12\x0c\n\x08\x44YNAMODB\x10\x04\x12\x0c\n\x08\x43OSMOSDB\x10\x05*\xa2\x01\n\x08\x44\x61taType\x12\x0b\n\x07INTEGER\x10\x01\x12\t\n\x05\x46LOAT\x10\x02\x12\x0b\n\x07\x42OOLEAN\x10\x03\x12\n\n\x06STRING\x10\x04\x12\n\n\x06\x44OUBLE\x10\x05\x12\x08\n\x04LONG\x10\x06\x12\r\n\tTIMESTAMP\x10\x07\x12\x08\n\x04\x44\x41TE\x10\x08\x12\t\n\x05SHORT\x10\t\x12\t\n\x05\x41RRAY\x10\n\x12\x07\n\x03MAP\x10\x0b\x12\n\n\x06\x42INARY\x10\x0c\x12\x0b\n\x07\x44\x45\x43IMAL\x10\r*4\n\tQueryMode\x12\x16\n\x12PRIMARY_KEY_LOOKUP\x10\x00\x12\x0f\n\x0bRANGE_QUERY\x10\x01\x42\x31\n\'com.databricks.proto.featurestorecommon\xa0\x01\x01\xe2?\x02\x10\x01')

_CLOUD = DESCRIPTOR.enum_types_by_name['Cloud']
Cloud = enum_type_wrapper.EnumTypeWrapper(_CLOUD)
_STORETYPE = DESCRIPTOR.enum_types_by_name['StoreType']
StoreType = enum_type_wrapper.EnumTypeWrapper(_STORETYPE)
_DATATYPE = DESCRIPTOR.enum_types_by_name['DataType']
DataType = enum_type_wrapper.EnumTypeWrapper(_DATATYPE)
_QUERYMODE = DESCRIPTOR.enum_types_by_name['QueryMode']
QueryMode = enum_type_wrapper.EnumTypeWrapper(_QUERYMODE)
AWS = 1
AZURE = 2
GCP = 3
AURORA_MYSQL = 1
SQL_SERVER = 2
MYSQL = 3
DYNAMODB = 4
COSMOSDB = 5
INTEGER = 1
FLOAT = 2
BOOLEAN = 3
STRING = 4
DOUBLE = 5
LONG = 6
TIMESTAMP = 7
DATE = 8
SHORT = 9
ARRAY = 10
MAP = 11
BINARY = 12
DECIMAL = 13
PRIMARY_KEY_LOOKUP = 0
RANGE_QUERY = 1


_ONLINESTOREFORSERVING = DESCRIPTOR.message_types_by_name['OnlineStoreForServing']
_MYSQLCONF = DESCRIPTOR.message_types_by_name['MySqlConf']
_SQLSERVERCONF = DESCRIPTOR.message_types_by_name['SqlServerConf']
_DYNAMODBCONF = DESCRIPTOR.message_types_by_name['DynamoDbConf']
_COSMOSDBCONF = DESCRIPTOR.message_types_by_name['CosmosDbConf']
_FEATUREDETAILS = DESCRIPTOR.message_types_by_name['FeatureDetails']
_PRIMARYKEYDETAILS = DESCRIPTOR.message_types_by_name['PrimaryKeyDetails']
_TIMESTAMPKEYDETAILS = DESCRIPTOR.message_types_by_name['TimestampKeyDetails']
_ONLINEFEATURETABLE = DESCRIPTOR.message_types_by_name['OnlineFeatureTable']
_FEATURETABLESFORSERVING = DESCRIPTOR.message_types_by_name['FeatureTablesForServing']
_MYSQLMETADATA = DESCRIPTOR.message_types_by_name['MySqlMetadata']
_SQLSERVERMETADATA = DESCRIPTOR.message_types_by_name['SqlServerMetadata']
_DYNAMODBMETADATA = DESCRIPTOR.message_types_by_name['DynamoDbMetadata']
_COSMOSDBMETADATA = DESCRIPTOR.message_types_by_name['CosmosDbMetadata']
_ONLINESTORE = DESCRIPTOR.message_types_by_name['OnlineStore']
_ONLINESTOREDETAILED = DESCRIPTOR.message_types_by_name['OnlineStoreDetailed']
_FEATURETABLESFORSAGEMAKERSERVING = DESCRIPTOR.message_types_by_name['FeatureTablesForSageMakerServing']
_ONLINEFEATURETABLEFORSAGEMAKERSERVING = DESCRIPTOR.message_types_by_name['OnlineFeatureTableForSageMakerServing']
_ONLINESTOREFORSAGEMAKERSERVING = DESCRIPTOR.message_types_by_name['OnlineStoreForSageMakerServing']
_FEATUREFUNCTIONPARAMETERINFO = DESCRIPTOR.message_types_by_name['FeatureFunctionParameterInfo']
_FEATUREFUNCTIONFORSERVING = DESCRIPTOR.message_types_by_name['FeatureFunctionForServing']
_FEATUREFUNCTIONSFORSERVING = DESCRIPTOR.message_types_by_name['FeatureFunctionsForServing']
OnlineStoreForServing = _reflection.GeneratedProtocolMessageType('OnlineStoreForServing', (_message.Message,), {
  'DESCRIPTOR' : _ONLINESTOREFORSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineStoreForServing)
  })
_sym_db.RegisterMessage(OnlineStoreForServing)

MySqlConf = _reflection.GeneratedProtocolMessageType('MySqlConf', (_message.Message,), {
  'DESCRIPTOR' : _MYSQLCONF,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.MySqlConf)
  })
_sym_db.RegisterMessage(MySqlConf)

SqlServerConf = _reflection.GeneratedProtocolMessageType('SqlServerConf', (_message.Message,), {
  'DESCRIPTOR' : _SQLSERVERCONF,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.SqlServerConf)
  })
_sym_db.RegisterMessage(SqlServerConf)

DynamoDbConf = _reflection.GeneratedProtocolMessageType('DynamoDbConf', (_message.Message,), {
  'DESCRIPTOR' : _DYNAMODBCONF,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.DynamoDbConf)
  })
_sym_db.RegisterMessage(DynamoDbConf)

CosmosDbConf = _reflection.GeneratedProtocolMessageType('CosmosDbConf', (_message.Message,), {
  'DESCRIPTOR' : _COSMOSDBCONF,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.CosmosDbConf)
  })
_sym_db.RegisterMessage(CosmosDbConf)

FeatureDetails = _reflection.GeneratedProtocolMessageType('FeatureDetails', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREDETAILS,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureDetails)
  })
_sym_db.RegisterMessage(FeatureDetails)

PrimaryKeyDetails = _reflection.GeneratedProtocolMessageType('PrimaryKeyDetails', (_message.Message,), {
  'DESCRIPTOR' : _PRIMARYKEYDETAILS,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.PrimaryKeyDetails)
  })
_sym_db.RegisterMessage(PrimaryKeyDetails)

TimestampKeyDetails = _reflection.GeneratedProtocolMessageType('TimestampKeyDetails', (_message.Message,), {
  'DESCRIPTOR' : _TIMESTAMPKEYDETAILS,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.TimestampKeyDetails)
  })
_sym_db.RegisterMessage(TimestampKeyDetails)

OnlineFeatureTable = _reflection.GeneratedProtocolMessageType('OnlineFeatureTable', (_message.Message,), {
  'DESCRIPTOR' : _ONLINEFEATURETABLE,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineFeatureTable)
  })
_sym_db.RegisterMessage(OnlineFeatureTable)

FeatureTablesForServing = _reflection.GeneratedProtocolMessageType('FeatureTablesForServing', (_message.Message,), {
  'DESCRIPTOR' : _FEATURETABLESFORSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureTablesForServing)
  })
_sym_db.RegisterMessage(FeatureTablesForServing)

MySqlMetadata = _reflection.GeneratedProtocolMessageType('MySqlMetadata', (_message.Message,), {
  'DESCRIPTOR' : _MYSQLMETADATA,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.MySqlMetadata)
  })
_sym_db.RegisterMessage(MySqlMetadata)

SqlServerMetadata = _reflection.GeneratedProtocolMessageType('SqlServerMetadata', (_message.Message,), {
  'DESCRIPTOR' : _SQLSERVERMETADATA,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.SqlServerMetadata)
  })
_sym_db.RegisterMessage(SqlServerMetadata)

DynamoDbMetadata = _reflection.GeneratedProtocolMessageType('DynamoDbMetadata', (_message.Message,), {
  'DESCRIPTOR' : _DYNAMODBMETADATA,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.DynamoDbMetadata)
  })
_sym_db.RegisterMessage(DynamoDbMetadata)

CosmosDbMetadata = _reflection.GeneratedProtocolMessageType('CosmosDbMetadata', (_message.Message,), {
  'DESCRIPTOR' : _COSMOSDBMETADATA,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.CosmosDbMetadata)
  })
_sym_db.RegisterMessage(CosmosDbMetadata)

OnlineStore = _reflection.GeneratedProtocolMessageType('OnlineStore', (_message.Message,), {
  'DESCRIPTOR' : _ONLINESTORE,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineStore)
  })
_sym_db.RegisterMessage(OnlineStore)

OnlineStoreDetailed = _reflection.GeneratedProtocolMessageType('OnlineStoreDetailed', (_message.Message,), {
  'DESCRIPTOR' : _ONLINESTOREDETAILED,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineStoreDetailed)
  })
_sym_db.RegisterMessage(OnlineStoreDetailed)

FeatureTablesForSageMakerServing = _reflection.GeneratedProtocolMessageType('FeatureTablesForSageMakerServing', (_message.Message,), {
  'DESCRIPTOR' : _FEATURETABLESFORSAGEMAKERSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureTablesForSageMakerServing)
  })
_sym_db.RegisterMessage(FeatureTablesForSageMakerServing)

OnlineFeatureTableForSageMakerServing = _reflection.GeneratedProtocolMessageType('OnlineFeatureTableForSageMakerServing', (_message.Message,), {
  'DESCRIPTOR' : _ONLINEFEATURETABLEFORSAGEMAKERSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineFeatureTableForSageMakerServing)
  })
_sym_db.RegisterMessage(OnlineFeatureTableForSageMakerServing)

OnlineStoreForSageMakerServing = _reflection.GeneratedProtocolMessageType('OnlineStoreForSageMakerServing', (_message.Message,), {
  'DESCRIPTOR' : _ONLINESTOREFORSAGEMAKERSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnlineStoreForSageMakerServing)
  })
_sym_db.RegisterMessage(OnlineStoreForSageMakerServing)

FeatureFunctionParameterInfo = _reflection.GeneratedProtocolMessageType('FeatureFunctionParameterInfo', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREFUNCTIONPARAMETERINFO,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureFunctionParameterInfo)
  })
_sym_db.RegisterMessage(FeatureFunctionParameterInfo)

FeatureFunctionForServing = _reflection.GeneratedProtocolMessageType('FeatureFunctionForServing', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREFUNCTIONFORSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureFunctionForServing)
  })
_sym_db.RegisterMessage(FeatureFunctionForServing)

FeatureFunctionsForServing = _reflection.GeneratedProtocolMessageType('FeatureFunctionsForServing', (_message.Message,), {
  'DESCRIPTOR' : _FEATUREFUNCTIONSFORSERVING,
  '__module__' : 'feature_store_serving_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureFunctionsForServing)
  })
_sym_db.RegisterMessage(FeatureFunctionsForServing)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\'com.databricks.proto.featurestorecommon\240\001\001\342?\002\020\001'
  _ONLINESTORE.fields_by_name['host']._options = None
  _ONLINESTORE.fields_by_name['host']._serialized_options = b'\030\001'
  _ONLINESTORE.fields_by_name['port']._options = None
  _ONLINESTORE.fields_by_name['port']._serialized_options = b'\030\001'
  _ONLINESTOREDETAILED.fields_by_name['host']._options = None
  _ONLINESTOREDETAILED.fields_by_name['host']._serialized_options = b'\030\001'
  _ONLINESTOREDETAILED.fields_by_name['port']._options = None
  _ONLINESTOREDETAILED.fields_by_name['port']._serialized_options = b'\030\001'
  _CLOUD._serialized_start=3991
  _CLOUD._serialized_end=4027
  _STORETYPE._serialized_start=4029
  _STORETYPE._serialized_end=4113
  _DATATYPE._serialized_start=4116
  _DATATYPE._serialized_end=4278
  _QUERYMODE._serialized_start=4280
  _QUERYMODE._serialized_end=4332
  _ONLINESTOREFORSERVING._serialized_start=75
  _ONLINESTOREFORSERVING._serialized_end=551
  _MYSQLCONF._serialized_start=553
  _MYSQLCONF._serialized_end=592
  _SQLSERVERCONF._serialized_start=594
  _SQLSERVERCONF._serialized_end=637
  _DYNAMODBCONF._serialized_start=639
  _DYNAMODBCONF._serialized_end=669
  _COSMOSDBCONF._serialized_start=671
  _COSMOSDBCONF._serialized_end=706
  _FEATUREDETAILS._serialized_start=708
  _FEATUREDETAILS._serialized_end=814
  _PRIMARYKEYDETAILS._serialized_start=816
  _PRIMARYKEYDETAILS._serialized_end=898
  _TIMESTAMPKEYDETAILS._serialized_start=900
  _TIMESTAMPKEYDETAILS._serialized_end=984
  _ONLINEFEATURETABLE._serialized_start=987
  _ONLINEFEATURETABLE._serialized_end=1341
  _FEATURETABLESFORSERVING._serialized_start=1343
  _FEATURETABLESFORSERVING._serialized_end=1431
  _MYSQLMETADATA._serialized_start=1433
  _MYSQLMETADATA._serialized_end=1476
  _SQLSERVERMETADATA._serialized_start=1478
  _SQLSERVERMETADATA._serialized_end=1525
  _DYNAMODBMETADATA._serialized_start=1527
  _DYNAMODBMETADATA._serialized_end=1593
  _COSMOSDBMETADATA._serialized_start=1595
  _COSMOSDBMETADATA._serialized_end=1670
  _ONLINESTORE._serialized_start=1673
  _ONLINESTORE._serialized_end=2226
  _ONLINESTOREDETAILED._serialized_start=2229
  _ONLINESTOREDETAILED._serialized_end=2836
  _FEATURETABLESFORSAGEMAKERSERVING._serialized_start=2838
  _FEATURETABLESFORSAGEMAKERSERVING._serialized_end=2954
  _ONLINEFEATURETABLEFORSAGEMAKERSERVING._serialized_start=2957
  _ONLINEFEATURETABLEFORSAGEMAKERSERVING._serialized_end=3339
  _ONLINESTOREFORSAGEMAKERSERVING._serialized_start=3342
  _ONLINESTOREFORSAGEMAKERSERVING._serialized_end=3532
  _FEATUREFUNCTIONPARAMETERINFO._serialized_start=3535
  _FEATUREFUNCTIONPARAMETERINFO._serialized_end=3666
  _FEATUREFUNCTIONFORSERVING._serialized_start=3669
  _FEATUREFUNCTIONFORSERVING._serialized_end=3888
  _FEATUREFUNCTIONSFORSERVING._serialized_start=3890
  _FEATUREFUNCTIONSFORSERVING._serialized_end=3989
# @@protoc_insertion_point(module_scope)
