# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feature_spec.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from mlflow.protos.scalapb import scalapb_pb2 as scalapb_dot_scalapb__pb2
from mlflow.protos import databricks_pb2 as databricks__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x66\x65\x61ture_spec.proto\x12\x12\x66\x65\x61turestorecommon\x1a\x15scalapb/scalapb.proto\x1a\x10\x64\x61tabricks.proto\"$\n\x14SourceDataColumnInfo\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x84\x01\n\x11\x46\x65\x61tureColumnInfo\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66\x65\x61ture_name\x18\x02 \x01(\t\x12\x12\n\nlookup_key\x18\x03 \x03(\t\x12\x13\n\x0boutput_name\x18\x04 \x01(\t\x12\x1c\n\x14timestamp_lookup_key\x18\x05 \x03(\t\"3\n\x0cInputBinding\x12\x11\n\tparameter\x18\x01 \x01(\t\x12\x10\n\x08\x62ound_to\x18\x02 \x01(\t\"u\n\x12OnDemandColumnInfo\x12\x10\n\x08udf_name\x18\x01 \x01(\t\x12\x38\n\x0einput_bindings\x18\x02 \x03(\x0b\x32 .featurestorecommon.InputBinding\x12\x13\n\x0boutput_name\x18\x03 \x01(\t\"\x87\x02\n\nColumnInfo\x12K\n\x17source_data_column_info\x18\x01 \x01(\x0b\x32(.featurestorecommon.SourceDataColumnInfoH\x00\x12\x44\n\x13\x66\x65\x61ture_column_info\x18\x02 \x01(\x0b\x32%.featurestorecommon.FeatureColumnInfoH\x00\x12G\n\x15on_demand_column_info\x18\x03 \x01(\x0b\x32&.featurestorecommon.OnDemandColumnInfoH\x00\x12\x15\n\x07include\x18\n \x01(\x08:\x04trueB\x06\n\x04info\"8\n\x10\x46\x65\x61tureTableInfo\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x10\n\x08table_id\x18\x02 \x01(\t\"-\n\x0c\x46unctionInfo\x12\x10\n\x08udf_name\x18\x01 \x01(\t\x12\x0b\n\x03md5\x18\x02 \x01(\t\"\x96\x02\n\x0b\x46\x65\x61tureSpec\x12\x35\n\rinput_columns\x18\x01 \x03(\x0b\x32\x1e.featurestorecommon.ColumnInfo\x12\x1d\n\x15serialization_version\x18\x02 \x01(\x05\x12\x14\n\x0cworkspace_id\x18\x03 \x01(\x03\x12$\n\x1c\x66\x65\x61ture_store_client_version\x18\x04 \x01(\t\x12:\n\x0cinput_tables\x18\x05 \x03(\x0b\x32$.featurestorecommon.FeatureTableInfo\x12\x39\n\x0finput_functions\x18\x06 \x03(\x0b\x32 .featurestorecommon.FunctionInfoBC\n\'com.databricks.proto.featurestorecommonB\x10\x46\x65\x61tureSpecProto\xa0\x01\x01\xe2?\x02\x10\x01')



_SOURCEDATACOLUMNINFO = DESCRIPTOR.message_types_by_name['SourceDataColumnInfo']
_FEATURECOLUMNINFO = DESCRIPTOR.message_types_by_name['FeatureColumnInfo']
_INPUTBINDING = DESCRIPTOR.message_types_by_name['InputBinding']
_ONDEMANDCOLUMNINFO = DESCRIPTOR.message_types_by_name['OnDemandColumnInfo']
_COLUMNINFO = DESCRIPTOR.message_types_by_name['ColumnInfo']
_FEATURETABLEINFO = DESCRIPTOR.message_types_by_name['FeatureTableInfo']
_FUNCTIONINFO = DESCRIPTOR.message_types_by_name['FunctionInfo']
_FEATURESPEC = DESCRIPTOR.message_types_by_name['FeatureSpec']
SourceDataColumnInfo = _reflection.GeneratedProtocolMessageType('SourceDataColumnInfo', (_message.Message,), {
  'DESCRIPTOR' : _SOURCEDATACOLUMNINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.SourceDataColumnInfo)
  })
_sym_db.RegisterMessage(SourceDataColumnInfo)

FeatureColumnInfo = _reflection.GeneratedProtocolMessageType('FeatureColumnInfo', (_message.Message,), {
  'DESCRIPTOR' : _FEATURECOLUMNINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureColumnInfo)
  })
_sym_db.RegisterMessage(FeatureColumnInfo)

InputBinding = _reflection.GeneratedProtocolMessageType('InputBinding', (_message.Message,), {
  'DESCRIPTOR' : _INPUTBINDING,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.InputBinding)
  })
_sym_db.RegisterMessage(InputBinding)

OnDemandColumnInfo = _reflection.GeneratedProtocolMessageType('OnDemandColumnInfo', (_message.Message,), {
  'DESCRIPTOR' : _ONDEMANDCOLUMNINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.OnDemandColumnInfo)
  })
_sym_db.RegisterMessage(OnDemandColumnInfo)

ColumnInfo = _reflection.GeneratedProtocolMessageType('ColumnInfo', (_message.Message,), {
  'DESCRIPTOR' : _COLUMNINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.ColumnInfo)
  })
_sym_db.RegisterMessage(ColumnInfo)

FeatureTableInfo = _reflection.GeneratedProtocolMessageType('FeatureTableInfo', (_message.Message,), {
  'DESCRIPTOR' : _FEATURETABLEINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureTableInfo)
  })
_sym_db.RegisterMessage(FeatureTableInfo)

FunctionInfo = _reflection.GeneratedProtocolMessageType('FunctionInfo', (_message.Message,), {
  'DESCRIPTOR' : _FUNCTIONINFO,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FunctionInfo)
  })
_sym_db.RegisterMessage(FunctionInfo)

FeatureSpec = _reflection.GeneratedProtocolMessageType('FeatureSpec', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESPEC,
  '__module__' : 'feature_spec_pb2'
  # @@protoc_insertion_point(class_scope:featurestorecommon.FeatureSpec)
  })
_sym_db.RegisterMessage(FeatureSpec)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\'com.databricks.proto.featurestorecommonB\020FeatureSpecProto\240\001\001\342?\002\020\001'
  _SOURCEDATACOLUMNINFO._serialized_start=83
  _SOURCEDATACOLUMNINFO._serialized_end=119
  _FEATURECOLUMNINFO._serialized_start=122
  _FEATURECOLUMNINFO._serialized_end=254
  _INPUTBINDING._serialized_start=256
  _INPUTBINDING._serialized_end=307
  _ONDEMANDCOLUMNINFO._serialized_start=309
  _ONDEMANDCOLUMNINFO._serialized_end=426
  _COLUMNINFO._serialized_start=429
  _COLUMNINFO._serialized_end=692
  _FEATURETABLEINFO._serialized_start=694
  _FEATURETABLEINFO._serialized_end=750
  _FUNCTIONINFO._serialized_start=752
  _FUNCTIONINFO._serialized_end=797
  _FEATURESPEC._serialized_start=800
  _FEATURESPEC._serialized_end=1078
# @@protoc_insertion_point(module_scope)
